//+----------------------------------------------------------------------------------+
//|                                                   SweetSpots.mq4                 |
//|                                                                                  |
//|                                                                                  |
//+----------------------------------------------------------------------------------+
#property copyright "Copyright Shimodax"
#property link      "http://www.strategybuilderfx.com"

/*------------------------------------------------------------------------------------
SDX-SweetSpots_v2:

Flexibility in line styles is increased.  You can now select to make the lines Solid,
Dash, Dot, or DashDotDot.  And when selecting Solid lines, you can select thickness.

SDX-SweetSpots_v3:

This upgrade allows users to "turn off" the sub-lines, while retaining their settings.

SDX-SweetSpots_v4:

This upgrade does away with the two fixed levels at the whole and half numbers, and
allows the user to input up to four different levels of choice.  By default, the 
first level is set to the whole number (input "0"), the second level is set to the
half number (input "50"), the third level is set to the quarter (input "25") and the
fourth level is set to three-quarters (input "75").  The user can change these.
The user can select for each line the line style, line thickness (for solid lines),
and the line color.  And now each line can be separately toggled on/off with the
"true/false" switch.  The Range selection feature recognizes that the current price
is in a base range bounded by upper and lower whole numbers.  A selection of "0" for 
"RangesAboveBelowBaseRange" will fill out this base range with the selected lines.
A selection of "1" for "RangesAboveBelowBaseRange" will also add a full range of the
lines both above and below the base range.  A selection of "2" will place two full
ranges of lines above and below the base range, etc.  As the price crosses whole 
number lines, the indicator appropriately deletes old and adds new ranges of lines.

                                                    -  Traderathome, November 23, 2008
SDX-SweetSpots_v4_MP6L:

MP6L = MultiPlatform - 6 line level inputs.  This upgrade adds the necessary
code for the indicator to also work with platforms that have unconventional
Point digits number (0.00001 and 0.001).  Also were added two more customizable
line levels which default to 66 and 33, as these are places where price usually 
stops and bounces.

                                                    -  Doblece, December 3, 2008 
                                                    
SDX-SweetSpots_v5: 

The base for this upgrade is the previous _v4_MP6L release.  Line labels are added
as an option in the indicator window, offering color, font size, font style, and
the ability to specify your own text to go with the label price.

                                                    -  Traderathome, December 5, 2008
                                                   
SDX-SweetSpots_5.2:

Adds On/Off switch for indicator and the ability to independently turn on/off the
price display for the line labels and the chart margin.  The position set for labels
(user adjustable) is relative to the chart verticle centerline for the current chart
density.  If the user subsequently increases or decreases the chart density, the 
selected position for labels automatically corrects with the next data tick. 

                                                    -  Traderathome, March 1, 2009                                                                                                                                                  
--------------------------------------------------------------------------------------*/  

#property indicator_chart_window

extern bool   Indicator_On?              = true;
extern int    RangesAboveBelowBaseRange  = 1;

extern bool   Margin_Prices?             = true;
extern bool   Line_Prices?               = false;
extern string TextinLabel                = "Psych Level";
extern int    Slide_Labels_LR_Decr_Incr  = 0;             //-# to move left, +# to move right
extern color  LabelColor                 = Gray;
extern int    Size_Font                  = 9;             //+ to increase size
extern int    Normal_Bold_Black_123      = 1;
extern bool   ShowLabels?                = true;

extern color  FirstLineColor             = Crimson;
extern bool   FirstLineOn                = true;
extern int    FirstLine0s_orEnd2Digits   = 0;
extern int    FirstLineStyle_01234       = 0;
extern int    FirstLineSolidThickness    = 1;

extern color  SecondLineColor            = RoyalBlue;
extern bool   SecondLineOn               = true;
extern int    SecondLineEnd2Digits       = 50;
extern int    SecondLineStyle_01234      = 2;
extern int    SecondLineSolidThickness   = 1;

extern color  ThirdLineColor             = LimeGreen;
extern bool   ThirdLineOn                = true;  
extern int    ThirdLineEnd2Digits        = 20;
extern int    ThirdLineStyle_01234       = 2;
extern int    ThirdLineSolidThickness    = 1;

extern color  FourthLineColor            = LimeGreen;
extern bool   FourthLineOn               = true;
extern int    FourthLineEnd2Digits       = 80;
extern int    ForthLineStyle_01234       = 2;
extern int    ForthLineSolidThickness    = 1;

extern color  FifthLineColor             = Olive;
extern bool   FifthLineOn                = true;
extern int    FifthLineEnd2Digits        = 66;
extern int    FifthLineStyle_01234       = 2;
extern int    FifthLineSolidThickness    = 1;

extern color  SixthLineColor             = Olive;
extern bool   SixthLineOn                = true;
extern int    SixthLineEnd2Digits        = 33;
extern int    SixthLineStyle_01234       = 2;
extern int    SixthLineSolidThickness    = 1;

double Poin;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{ 
   //Checking for unconventional Point digits number
   if (Point == 0.00001) Poin = 0.0001; //5 digits
   else if (Point == 0.001) Poin = 0.01; //3 digits
   else Poin = Point; //Normal
   return(0);
}
//+------------------------------------------------------------------+
//| Custom indicator Deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()  
{
   int obj_total= ObjectsTotal();  
   for (int j= obj_total; j>=0; j--) 
      {
      string name= ObjectName(j);    
      if (StringSubstr(name,0,12)=="[SweetSpot] "){ObjectDelete(name);}
      }  
   return(0);
}

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{
   if (Indicator_On? == false) { return(0);} 
   //count the chars Objects, then for that count check and clear any entitled "[SweetSpot]"
   int obj_total= ObjectsTotal();  
   for (int j= obj_total; j>=0; j--) 
      {
      string name= ObjectName(j);    
      if (StringSubstr(name,0,12)=="[SweetSpot] "){ObjectDelete(name);}
      }   

   //space out updates
     //static datetime timelastupdate= 0; static datetime lasttimeframe= 0;     
     //if (CurTime()-timelastupdate < 5 && Period()==lasttimeframe) return (0);

   //initialize needed variables
   int i, ssp, ssp1;  //choosing "int" instead of "double" drops decimal portion of value
   double linelevel, linestyle ,linewidth; color linecolor;
   double d1=Bid;  //Comment(DoubleToStr(NormalizeDouble(d1,2),2));
   ssp1= Bid;  //bid w/o decimal, 117 instead of 117.66
   ssp1= Bid / Poin;  //Point restores bid integers w/o the decimal, 11766   
   int u1 = FirstLine0s_orEnd2Digits; //initializing last two digits user entered
   int u2 = SecondLineEnd2Digits;
   int u3 = ThirdLineEnd2Digits;
   int u4 = FourthLineEnd2Digits;
   int u5 = FifthLineEnd2Digits;
   int u6 = SixthLineEnd2Digits;      
   int c1=ssp1%100;  
   double LA, LB, LN; LN=100-c1; LB=c1; LA=LN; //if (c1<50)LB=c1;LA=LN;//if (c1>+50)LB=c1;LA=LN;Comment(LB);

   //calculate line levels for user specified # of ranges
   ssp=ssp1; //start ssp at reference bid
   for (i= -((100*(RangesAboveBelowBaseRange))+LB); i<=((100*(RangesAboveBelowBaseRange))+LA); i++)   
      {
         ssp=ssp1+i;  //Comment(i+"  "+ssp);
         c1=ssp%100;  //Comment(c1);
         if (c1==u1 && FirstLineOn == true)
            {           
            linestyle = FirstLineStyle_01234;
            linecolor = FirstLineColor;
            linewidth = FirstLineSolidThickness;
            linelevel = ssp*Poin;
            }
         if (c1==u2 && SecondLineOn == true)
            {           
            linestyle = SecondLineStyle_01234;
            linecolor = SecondLineColor;
            linewidth = SecondLineSolidThickness;
            linelevel = ssp*Poin;
            }
         if (c1==u3 && ThirdLineOn == true)
            {           
            linestyle = ThirdLineStyle_01234;
            linecolor = ThirdLineColor;
            linewidth = ThirdLineSolidThickness;
            linelevel = ssp*Poin;
            }
         if (c1==u4 && FourthLineOn == true)
            {           
            linestyle = ForthLineStyle_01234;
            linecolor = FourthLineColor;
            linewidth = ForthLineSolidThickness;
            linelevel = ssp*Poin;
            }
         if (c1==u5 && FifthLineOn == true)
            {           
            linestyle = FifthLineStyle_01234;
            linecolor = FifthLineColor;
            linewidth = FifthLineSolidThickness;
            linelevel = ssp*Poin;
            }
         if (c1==u6 && FifthLineOn == true)
            {           
            linestyle = SixthLineStyle_01234;
            linecolor = SixthLineColor;
            linewidth = SixthLineSolidThickness;
            linelevel = ssp*Poin;
            }

         //branch to subroutine to draw lines
         SetLevel(DoubleToStr(linelevel,Digits), linelevel,  linecolor, linestyle, linewidth, Time[10]);
         
         //branch to subroutine to draw labels
         if(ShowLabels? == true){SetLabel(DoubleToStr(linelevel,Digits), linelevel, LabelColor);}
      }
   return(0);
}

//+------------------------------------------------------------------+
//| Sub-routine to draw the lines                                    |
//+------------------------------------------------------------------+
void SetLevel(string text, double linelevel, color linecolor, int linestyle, int linewidth, datetime startofday)
{
   int digits=Digits;  
   int AA = WindowFirstVisibleBar(); 
   string linename= "[SweetSpot] " + text + " Line",pricelabel;   
   // create or move the horizontal line 
   int Z= OBJ_TREND;
   if(  Margin_Prices? == true){Z = OBJ_HLINE;}    
   if (ObjectFind(linename) != 0)
      {
      ObjectCreate(linename,  Z, 0, AA, linelevel, Time[0], linelevel); 
      ObjectSet(linename, OBJPROP_STYLE, linestyle);     
      ObjectSet(linename, OBJPROP_COLOR, linecolor);
      ObjectSet(linename, OBJPROP_WIDTH, linewidth);
      }
   else 
      {
      ObjectMove(linename, 0, AA, linelevel);
      ObjectMove(linename, 1, Time[0], linelevel);
      }
}

//+------------------------------------------------------------------+
//| Sub-routine to draw the line labels                              |                                                        
//+------------------------------------------------------------------+
void SetLabel(string text, double linelevel, color Color_Labels)
{
   int ml = (WindowFirstVisibleBar() /2) - Slide_Labels_LR_Decr_Incr; 
   string Font;
   string name = "[SweetSpot] " + text + " Label";
   string TEXT = TextinLabel; 
   if (Line_Prices? == true) {TEXT = TEXT + ":  "+DoubleToStr(linelevel, Digits);} 
 
   if(ObjectFind(name) != 0)
      {
      if (Normal_Bold_Black_123 == 2){Font = "Arial Bold";}
      if (Normal_Bold_Black_123 >= 3){Font = "Arial Black";}
      ObjectCreate(name, OBJ_TEXT, 0, Time[ml], linelevel);       
      //ObjectSet(name, OBJPROP_FONTSIZE, Size_Font); 
      //ObjectSet(name, OBJPROP_COLOR, Color_Labels );
      ObjectSetText(name, TEXT, Size_Font, Font, Color_Labels);
      }
      else
      {
      ObjectMove(name, 0, Time[ml], linelevel);
      }  
}

//+-----------------End Program--------------------------------------+      